Set         sp slots plus initial
            s(sp) slots            
            c colors
            sa(sp,c) slot assignment
            pp(sp,c) pre placed balls 
Scalar      n        number of commercial in this batch
            nx       total number of commercial
            rlim     time limit
            rusd     time used
            sobj     objective of flow model
            bnd      bound from flow model
            bcnt     batch counter
Parameter   nc(c)    number of colors
            minc(c)  minimum balls of colors placed
            dc(c)    ideal distance; 

$gdxin solvein.gdx
$load sp c nc pp rlim dc n nx minc bcnt

Alias (sp,i,j);

s(sp+1) = ord(sp) <= n;

* Build network
Set         net(c,i,j) network connecting slots
Parameter   dev(c,i,j) deviation of distance i and j to ideal distance
Scalar      isf initial span factor /2/
            msf middle span factor range /%msf%/
            bst bost factor for colors with few balls /%bst%/
            sf  span factor;

$if set sparse $goto dosparse

* Dense graph
loop(c$nc(c),
  loop(i$(ord(i)<=n+1),
    net(c,i,j)$(ord(j)>ord(i) and ord(j)<=n+1) = yes;
  )
);
$goto contnet

$label dosparse
loop(c$nc(c),
  net(c,'0',j)$(ord(j)>1 and ord(j)<=min(n+1,1+isf*dc(c))) = yes;
  sf = msf*(1+bst$(nc(c)/nx<0.05)) ;
  if (sf*dc(c) < 20, sf = 20/dc(c));
  loop(i$(ord(i)>1 and ord(i)<=n+1),
    net(c,i,j)$(ord(j)>max(ord(i),ord(i)+(1-sf)*dc(c)) and ord(j)<=min(n+1,ord(i)+(1+sf)*dc(c))) = yes;
  )
);

$label contnet
* The arcs back to the source
net(c,s,'0') = yes;

dev(net(c,i,j))$(ord(i)>1 and ord(j)>1) = abs(ord(j)-ord(i) - dc(c));

* Model
Variable f(c,i,j) flow variable
         p(c,i)   placement variable
         obj      objective varibale

Binary variable p;
Positive variable f;

Equation bal(c,i)     flow balance equation
         balinit(c)   flow balance for node 0
         defopen(c,i) close slots for different colors
         defsump(c)   number of open slots
         covslot(i)   slot cover requirement
         defobj       objective function;

bal(c,s)$minc(c)..  
         sum(net(c,j,s), f(net)) - sum(net(c,s,j), f(net)) =e= 0;

balinit(c)$minc(c)..
         sum(net(c,'0',j), f(net)) =e= 1;

defopen(c,s)$minc(c)..
         sum(net(c,s,j), f(net)) =e= p(c,s);

defsump(c)$minc(c)..
         sum(s, p(c,s)) =e= minc(c);

covslot(s)..
         sum(c$minc(c), p(c,s)) =e= 1;

defobj.. obj =e= sum(net, dev(net)*f(net));

model p3 /all/;

option optcr=0, mip=cplex, iterlim=10000000;

p3.reslim  = rlim;

* Try priorities
p3.prioropt = 1;
p.prior(c,s) = minc(c);

p.lo(c,s)$pp(s,c)   = 1;

* Use CPLEX barrier
$echo startalg 4  > cplex.opt
p3.optfile = 1;

$if set sparse $goto skiplint
$if not exist solveout%curid%.gdx $goto skiplint
$gdxin solveout%curid%
$load sa
$echo mipstart 1  >> cplex.opt

p.l(c,s(sp+1)) = sa(sp,c);

$label skiplint

solve p3 min obj using mip;

abort$(p3.modelstat <> 1 and p3.modelstat <> 8) 'solve did not return a solution';

sa(sp,c) = round(p.l(c,sp+1));


sobj = obj.l;
bnd  = p3.objest;
rusd = p3.resusd;

parameter rep;
alias (sp,ss);
rep(sp,c) = sum(ss$(ord(ss)<=ord(sp)), p.l(c,ss));
display rep;